import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/common.dart';
import 'package:wowit/Interface/iWatting.dart';
import 'package:wowit/firebaseService.dart';
import 'package:wowit/message.dart';
import 'package:wowit/sqlService.dart';
import 'package:wowit/methods.dart';
import 'package:flutter/material.dart';

abstract class IAuthController {
  Future initSetting(Map<String, List<Map<String, Object>>> data);
}

class AuthController extends BaseController {
  final FirebaseAuth auth = FirebaseAuth.instance;
  final DatabaseReference db = FirebaseDatabase().reference();
  IAuthController action;
  Rx<User> _firebaseUser = Rx<User>(null);

  User get user => _firebaseUser.value;
  bool isInProcess = false;

  updateStatusButton() {
    isInProcess = true;
    update();
  }

  @override
  // ignore: must_call_super
  void onInit() {
    _firebaseUser.bindStream(auth.authStateChanges());
  }

  Future initAccountModel(bool triesLogin) async {
    String uidPhone = auth.currentUser.uid;
    String phoneNumber = auth.currentUser.phoneNumber;
    Common.uidPhone = auth.currentUser.uid;
    print('initAccountModel with uid:' + uidPhone);
    if (uidPhone == null || uidPhone == "") return;

    // set data Realtime Database
    //tam thoi bo, chua valid firebase
    if(Common.USE_ValidFirebase)
    {
      await db
          .child('tblAccount/$uidPhone')
          .set({"phone": phoneNumber, "uidPhone": uidPhone}).then((value) async {
        print('==========Thành công==========');
      });
    }
    
      var rsData = await SQLService.execute("sp_getLogin",
          params: {"@uidPhone": uidPhone, "@phone": phoneNumber});
      var data = Methods.getList(rsData, 'info');
      print('data count :' + data.length.toString());

      if (data.length == 0) {
        //account not exists -> create new account
        await SQLService.execute("sp_CreateAccount",
            params: {"@uidPhone": uidPhone, '@phone': phoneNumber});

        if (triesLogin) {
          await initAccountModel(false);
        }
      }
  }

  String strVerificationId = '';

  Future<void> signInWithPhone(
      final String phone, StatefulWidget screenOTP, Iwatting iwatting) async {
    this.updateStatusButton();

    iwatting.showWatting();

    final String phoneNumber = Methods.preparePhone(phone);
    auth.verifyPhoneNumber(
        phoneNumber: phoneNumber,
        timeout: Duration(seconds: 0),
        verificationCompleted: (AuthCredential authCredential) async {},
        verificationFailed: (FirebaseAuthException exception) {
          handleExceptionAuth(exception, iwatting);
          return '';
        },
        codeSent: (String verificationId, [int forceResendingToken]) async {
          iwatting.hideWatting();
          if (verificationId == '') {
            handleExceptionAuth(
                new Exception('VerificationId is empty'), iwatting);
            return;
          }
          iwatting.hideWatting();
          //show screen otp
          strVerificationId = verificationId;
          Get.to(() => screenOTP);
        },
        codeAutoRetrievalTimeout: (String verificationId) {
          iwatting.hideWatting();
          strVerificationId = verificationId;
        });
  }

  void verifyOTP(String otp, Widget homeScreen, Iwatting iwatting,
      HandleException handleException) {
    updateStatusButton();
    iwatting.showWatting();

    var _credential = PhoneAuthProvider.credential(
        verificationId: strVerificationId, smsCode: otp);
    auth.signInWithCredential(_credential).then((UserCredential result) async {
      await initAccountModel(true);
      iwatting.hideWatting();
      clearData();
      this.isInProcess = false;
      Get.offAll(() => homeScreen);
    }).catchError((e) {
      iwatting.hideWatting();

      if (handleException.handleExceptionAuth(e)) {
        handleExceptionAuth(e, iwatting);
      }
      this.isInProcess = false;
      this.update();
      return "error";
    });
  }

  void handleExceptionAuth(dynamic e, Iwatting iwatting) {
    iwatting.hideWatting();
    this.isInProcess = false;
    this.update();
    print('handleExceptionAuth :' + e.toString());
    if (e.toString().contains('The SMS code has expired')) {
      theSmsOTPTimeOut();
    } else if (e.toString().contains('Timed out waiting for SMS')) {
      theSMSCodeHasExpried();
    } else if (e.toString().contains(
        'We have blocked all requests from this device due to unusual activity')) {
      theBlockRequest();
    } else if (e.toString().contains(
        'The sms verification code used to create the phone auth credential is invalid')) {
      theSMSOTPInValid();
    } else {
      Get.snackbar(
        Message.getMessage('REGISTER_THONG_BAO'),
        e.toString(),
        snackPosition: SnackPosition.TOP,
        duration: new Duration(milliseconds: 10000),
      );
    }
  }

  void theSMSOTPInValid() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_OPT_IN_VALID'),
      snackPosition: SnackPosition.TOP,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theSmsOTPTimeOut() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_TIME_OUT'),
      snackPosition: SnackPosition.TOP,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theBlockRequest() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_BLOCK_REQUEST'),
      snackPosition: SnackPosition.TOP,
      duration: new Duration(milliseconds: 10000),
    );
  }

  void theSMSCodeHasExpried() {
    Get.snackbar(
      Message.getMessage('REGISTER_THONG_BAO'),
      Message.getMessage('REGISTER_THE_SMS_HAS_EXPIRED'),
      snackPosition: SnackPosition.TOP,
      duration: new Duration(milliseconds: 10000),
    );
  }

  Future<Map<String, List<Map<String, Object>>>> getLogin() async {
    var uidPhone = FirebaseAuth.instance.currentUser?.uid;
    var phone = FirebaseAuth.instance.currentUser?.phoneNumber;
    if (uidPhone == null || phone == null) return null;
    Common.uidPhone = uidPhone;
    print("getLogin uidPhone: " + uidPhone + " , phone: " + phone);
    var rs = await SQLService.execute('sp_getLogin',
        params: {'phone': phone, 'uidPhone': uidPhone});
    if (rs.isEmpty) {
      FirebaseService.logout();
    }
    var data = Methods.getList(rs, 'info');
    if (data.length > 0) {
      if (Methods.getString(data[0], "messageId").isNotEmpty) {
        _widgetDialogThongBao();
      }
      if (Methods.getInt(data[0], "ID") > 0) {
        return rs;
      } else {
        FirebaseService.logout();
        return null;
      }
    }
    return rs;
  }

  void requestPermissionFireBaseMessage(String notifyTokenId) async {
    //REQUEST PERMISSION
    if (await FirebaseService.initializeFirebaseMessaging()) {
      await FirebaseService.initNotifyToken(notifyTokenId);
    }
  }

  void _widgetDialogThongBao({String keyMeesage}) {
    Future.delayed(Duration(seconds: 1), () async {
      showDialog(
          context: Get.overlayContext,
          builder: (context) {
            return AlertDialog(
                insetPadding: EdgeInsets.all(50),
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.all(Radius.circular(10.0))),
                contentPadding: EdgeInsets.zero,
                clipBehavior: Clip.antiAliasWithSaveLayer,
                content: Container(
                  height: 100,
                  child: Column(
                    children: [
                      Expanded(
                          child: Center(
                              child: Text(Message.getMessage(
                                  keyMeesage ?? 'TAI_KHOAN_BI_VO_HIEU_HOA')))),
                      SizedBox(
                        width: 8,
                      ),
                      ElevatedButton(
                        onPressed: () => Get.back(),
                        child: Text(
                          Message.getMessage("TAI_KHOAN_CLOSE"),
                          style: TextStyle(color: Colors.white),
                        ),
                        style: ButtonStyle(backgroundColor:
                            MaterialStateProperty.resolveWith<Color>((state) {
                          return Common.colorAppbar;
                        })),
                      ),
                      SizedBox(
                        width: 8,
                      ),
                    ],
                  ),
                ));
          });
    });
  }

  @override
  Future<Map<String, List<Map<String, Object>>>> loadData() async {
    var dt = await this.getLogin();
    if (action != null) {
      await action.initSetting(dt);
    }
    return dt;
  }
}
