// import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'sqlService.dart';
import 'dart:io';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/material.dart';
import 'package:path_provider/path_provider.dart';

class FirebaseService {

  static Directory _tempDir;    

  ///lấy hình ảnh tử firebase chuyển thành file
  static Future<File> getImageFile({@required String storagePath}) async {
    if(_tempDir == null){
      _tempDir = await getTemporaryDirectory();
    }
    final fileName = storagePath.split('/').last;
    final file = File('${_tempDir.path}/$fileName');
    if (storagePath == '') return null;
    if (!file.existsSync()) {
      try {
        file.create(recursive: true);
         await FirebaseStorage.instance.ref(storagePath).writeToFile(file);
         return file;
      } catch (e) {
        await file.delete(recursive: true);
        return null;
      }
    }else {
      int sizeFile = await file.length();
      if(sizeFile<=0){
        await FirebaseStorage.instance.ref(storagePath).writeToFile(file);
        return file;
      }
    }
    return file;
  }

  ///hiển thị hình ảnh lấy từ firebase
  static Widget getWidgetImageFirebase(
      {@required String storagePath,
      double width ,
      double height,
      BoxFit boxFit,Color color,
      Widget widgetError,
      Widget widgetLoadding,
      Key key, double scale = 1.0,
      String semanticLabel,   bool excludeFromSemantics = false,
      Animation  opacity,   BlendMode colorBlendMode,
      AlignmentGeometry alignment = Alignment.center,
      ImageRepeat repeat = ImageRepeat.noRepeat,
      Rect centerSlice,
      bool matchTextDirection = false, 
      bool gaplessPlayback = false,
      bool isAntiAlias = false,
      FilterQuality filterQuality = FilterQuality.low,
      int cacheWidth,
      int cacheHeight,
      Color colorWaiting,
      Color colorError
      }) {
    return FutureBuilder<File>(
      future: getImageFile(storagePath: storagePath),
      builder: (context, snapshot) {
        if(snapshot.connectionState == ConnectionState.waiting){
          return widgetLoadding??_waitingView(width,height,colorWaiting: colorWaiting??Colors.grey[100]);
        }
        if (snapshot.data == null) {
          return _waitingView(width,height,colorWaiting: colorError??Colors.grey[100]);
        }
        return Image.file(
          snapshot.data,
          key:key,
          width: width,
          height: height,
          fit: boxFit,
          color: color,
          scale:scale,
          errorBuilder:(context, objeject, s){
              return widgetError??_waitingView(width,height);
          },
          semanticLabel:semanticLabel,
          excludeFromSemantics:excludeFromSemantics,
          opacity:opacity,
          colorBlendMode:colorBlendMode,
          alignment:alignment,
          repeat:repeat,centerSlice:centerSlice,
          matchTextDirection:matchTextDirection,
          gaplessPlayback:gaplessPlayback,
          isAntiAlias:isAntiAlias,
          filterQuality:filterQuality,
          cacheWidth:cacheWidth,
          cacheHeight:cacheHeight
        );
      },
      initialData: null,
    );
  }
  static Widget _waitingView(double width,double height,{Color colorWaiting}){
    return Container(
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(10),
        color: colorWaiting??Colors.grey[100]),
      width: width,
      height: height,
    );
  }
  // static final lstCommonCollection = ['Account', 'Language'];

  // static CollectionReference getCollection(String collectionName) {
  //   if (lstCommonCollection.contains(collectionName)) {
  //     return FirebaseFirestore.instance.collection(collectionName);
  //   }
  //   return FirebaseFirestore.instance
  //       .collection(Common.mainCollection)
  //       .doc(Common.mainDocument)
  //       .collection(collectionName);
  // }

  // static Query getQuery(String collectionName,
  //     {Map<String, Object> conditions,
  //       Map<String, bool> orderBy,
  //       int limit = -1}) {
  //   if (limit == null) {
  //     limit = -1;
  //   }

  //   Query collection = FirebaseService.getCollection(collectionName);
  //   try {
  //     if (conditions != null && conditions.length > 0) {
  //       conditions.forEach((key, value) {
  //         if (value is Map<String, Object>) {
  //           value.forEach((k, v) {
  //             var _type = value['type'];
  //             var _value = value['value'];
  //             if (_type == '==')
  //               collection = collection.where(key, isEqualTo: _value);
  //             else if (_type == '!=')
  //               collection = collection.where(key, isNotEqualTo: _value);
  //             else if (_type == '<')
  //               collection = collection.where(key, isLessThan: _value);
  //             else if (_type == '<=')
  //               collection = collection.where(key, isLessThanOrEqualTo: _value);
  //             else if (_type == '>')
  //               collection = collection.where(key, isGreaterThan: _value);
  //             else if (_type == '>=')
  //               collection =
  //                   collection.where(key, isGreaterThanOrEqualTo: _value);
  //             else if (_type == 'arrayContains')
  //               collection = collection.where(key, arrayContains: _value);
  //             else if (_type == 'arrayContainsAny')
  //               collection = collection.where(key, arrayContainsAny: _value);
  //             else if (_type == 'whereIn')
  //               collection = collection.where(key, whereIn: _value);
  //             else if (_type == 'whereNotIn')
  //               collection = collection.where(key, whereNotIn: _value);
  //           });
  //         } else {
  //           collection = collection.where(key, isEqualTo: value);
  //         }
  //       });
  //     }

  //     if (orderBy != null && orderBy.length > 0) {
  //       orderBy.forEach((key, value) {
  //         collection = collection.orderBy(key, descending: value);
  //       });
  //     }
  //     if (limit > 0) {
  //       collection = collection.limit(limit);
  //     }
  //   } catch (e) {
  //     print('getCollection : ' + e.toString());
  //   }

  //   return collection;
  // }

  // static int getNewDocumentID() {
  //   DateTime dt = DateTime.now();
  //   return dt.difference(DateTime.utc(2020, 1, 1)).inMilliseconds;
  // }

  // static Map getNewDataTemplate(Map<String, Object> data, int uid) {
  //   int documentID = getNewDocumentID();
  //   data["documentID"] = documentID;
  //   if(uid == 9999) uid = documentID;
  //   data["uid"] = uid;
  //   data["statusData"] = 1;
  //   data["createDate"] = FieldValue.serverTimestamp();
  //   return data;
  // }

  // static Map getModified(Map<String, Object> data, int uid) {
  //   data["modifiedUid"] = uid;
  //   data["modifiedDate"] = FieldValue.serverTimestamp();
  //   return data;
  // }

  // static Future<int> setData(
  //     Map<String, Object> data, String colllectionPath, int uid) async {
  //   bool isNew;
  //   if (data.containsKey('documentID')) {
  //     isNew = false;
  //     data = getModified(data, uid);
  //   } else {
  //     isNew = true;
  //     data = getNewDataTemplate(data, uid);
  //   }
  //   int documentID = Methods.toInt(data['documentID'].toString());
  //   if (isNew) {
  //     await getCollection(colllectionPath)
  //         .doc(documentID.toString())
  //         .set(data, SetOptions(merge: true));
  //   } else {
  //     await getCollection(colllectionPath)
  //         .doc(documentID.toString())
  //         .update(data);
  //   }
  //   return documentID;
  // }

  // static Future delete(int documentID, String colllectionPath, int uid) async {
  //   Map<String, Object> data = {};
  //   data['statusData'] = 4; // da xoa
  //   data['deletedUid'] = uid;
  //   data['deletedDate'] = FieldValue.serverTimestamp();

  //  await getCollection(colllectionPath)
  //       .doc(documentID.toString())
  //       .set(data, SetOptions(merge: true));
  // }

  static Future logout() async {
    try {
      if (FirebaseAuth.instance.currentUser == null) return;
      String uidPhone = FirebaseAuth.instance.currentUser.uid;
      String phone = FirebaseAuth.instance.currentUser.phoneNumber;
      await SQLService.execute("sp_Logout",
          params: {"phone": phone, "uidPhone": uidPhone});
      await FirebaseAuth.instance.signOut();
    } catch (e) {
      print('logout: ' + e.toString());
    }
  }

  static Future initNotifyToken(String currentToken) async {
    try {
      if (FirebaseAuth.instance.currentUser == null) return;
      FirebaseMessaging messaging = FirebaseMessaging.instance;
      await messaging.getToken().then((token) async {
        if (currentToken == token) return;
        String uidPhone = FirebaseAuth.instance.currentUser.uid;
        String phone = FirebaseAuth.instance.currentUser.phoneNumber;
        await SQLService.execute("sp_FirebaseMessaging", params: {
          "phone": phone,
          "uidPhone": uidPhone,
          'notifyToken': token
        });
      });
    } catch (e) {
      print('login; ' + e.toString());
    }
  }

  //REQUEST PERMISSION NOTIFY
  static Future<bool> initializeFirebaseMessaging() async {
    final FirebaseMessaging firebaseMessaging = FirebaseMessaging.instance;
    var settings = await firebaseMessaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
    if (settings.authorizationStatus == AuthorizationStatus.authorized) {
      print('User granted permission FirebaseMessaging');
      return true;
    } else {
      print('User declined or has not accepted permission FirebaseMessaging');
      return false;
    }
  }
}
